/*++
  This file contains 'Framework Code' and is licensed as such
  under the terms of your license agreement with Intel or your
  vendor.  This file may not be modified, except as allowed by
  additional terms of your license agreement.
--*/
/** @file
  Serial I/O Port library functions with no library constructor/destructor

  Copyright (c) 2012 Intel Corporation.<BR>
  All rights reserved.  This software and associated documentation
  (if any) is furnished under a license and may only be used or
  copied in accordance with the terms of the license.  Except as
  permitted by such license, no part of this software or
  documentation may be reproduced, stored in a retrieval system, or
  transmitted in any form or by any means without the express written
  consent of Intel Corporation.

  Module Name:  SerialPortLib.c

**/

#include <PiPei.h>

#include <Library/SerialPortLib.h>
#include <Library/PcdLib.h>
#include <Library/IoLib.h>
#include <Library/PciLib.h>
#include <Protocol/SerialIo.h>

#include "SerialPortLib.h"

#define HIGH_8(Address16) ((UINT8) (((UINT16) Address16) >> 8))
#define LOW_8(Address16) ((UINT8) ((UINT16) Address16 & 0x00FF))

//
// Define the maximum debug and assert message length that this library supports 
//
#define MAX_DEBUG_MESSAGE_LENGTH  0x100

#define IO_DEC_DEFAULT  (0x3F0F0010)
#define PCI_IDX         0xCF8
#define PCI_DAT         0xCFC
#define LPC_IO_DEC      (0x80)
#define PCI_LPC_BASE    (0x8000F800)
#define PCI_LPC_REG(x)  (PCI_LPC_BASE + (x))

RETURN_STATUS
EnableLpcIoDecode(
  VOID
  )
/*++

Routine Description:

  Enable legacy decoding on ICH6

Arguments:

  none
  
Returns:

  EFI_SUCCESS     Always returns success.

--*/
{  
  //
  // Choose I/O base for legacy device and
  // enable I/O decoding for legacy device
  // For size consideration, just combine two operations.
  //
  IoWrite32 (PCI_IDX,  PCI_LPC_REG(LPC_IO_DEC));
  IoWrite32 (PCI_DAT,  IO_DEC_DEFAULT);

  return  0;
}

/**
  The constructor function initialize the Serial Port Library

  @retval EFI_SUCCESS   The constructor always returns RETURN_SUCCESS.

**/
VOID
EFIAPI
SioInitSerialPort (
  VOID
  )
{
  UINT8     Data = 0;

  //
  // Enable LPC decode
  // 
  EnableLpcIoDecode();
  
  //IoWrite8 (SIO_CONFIG_PORT, 0x55);
  
  //
  // Check if SIO is National
  //
  IoWrite8 (SIO_INDEX_PORT, 0x20);
  Data = IoRead8(SIO_DATA_PORT);
  if (Data == 0xF0) {
    IoWrite8 (SIO_INDEX_PORT, 0x2D);
    IoWrite8 (SIO_DATA_PORT,  0x00);
    IoWrite8 (SIO_INDEX_PORT, 0x29);
    IoWrite8 (SIO_DATA_PORT,  0x80);
  }

  //
  // Initialize Serial controller
  //
  //IoWrite8 (SIO_INDEX_PORT, 0x22);

  //IoWrite8 (SIO_DATA_PORT, 0x19);

  IoWrite8 (SIO_INDEX_PORT, 0x07);
  
  IoWrite8 (SIO_DATA_PORT, 0x03);
  
  IoWrite8 (SIO_INDEX_PORT, R_SIO_BASE_ADDRESS_LOW);
  
  IoWrite8 (SIO_DATA_PORT, LOW_8 ((UINT16)PcdGet64 (PcdSerialRegisterBase)));
  
  IoWrite8 (SIO_INDEX_PORT, R_SIO_BASE_ADDRESS_HIGH);
  
  IoWrite8 (SIO_DATA_PORT, HIGH_8 ((UINT16)PcdGet64 (PcdSerialRegisterBase)));
  
  IoWrite8 (SIO_INDEX_PORT, R_SIO_PRIMARY_INTERRUPT_SELECT);
  
  IoWrite8 (SIO_DATA_PORT, 0x04);
  
  IoWrite8 (SIO_INDEX_PORT, R_SIO_ACTIVATE);
  
  IoWrite8 (SIO_DATA_PORT, 0x01);

  //IoWrite8 (SIO_CONFIG_PORT, 0xAA);
}

/**

  Programmed hardware of Serial port.

  @return Always return EFI_SUCCESS.

**/
EFI_STATUS
EFIAPI
SerialPortInitialize (
  VOID
  )
{

  UINT32  Divisor;
  UINT8   OutputData;

  //
  // Some init is done by the platform status code initialization.
  //
  SioInitSerialPort ();

  //
  // Calculate divisor for baud generator
  //
  if (PcdGet64 (PcdUartDefaultBaudRate) != 0) {
    Divisor = (UINT32)(115200 / (UINTN)(PcdGet64 (PcdUartDefaultBaudRate)));
  } else {
    Divisor = 1;
  }

  //
  // Set communications format
  //
  switch (PcdGet8(PcdUartDefaultDataBits)) {
  case 5:
    OutputData = 0;
    break;

  case 6:
    OutputData = 1;
    break;

  case 7:
    OutputData = 2;
    break;

  case 8:
    OutputData = 3;
    break;

  default:
    OutputData = 0;
    break;
  }

  switch (PcdGet8(PcdUartDefaultStopBits)) {
  case OneFiveStopBits:
  case TwoStopBits:
    OutputData |= 0x04;
    break;

  case OneStopBit:
  default:
    OutputData &= 0xFB;
    break;
  }

  switch (PcdGet8(PcdUartDefaultParity)) {
  case EvenParity:
    OutputData |= 0x18;
    break;

  case OddParity:
    OutputData |= 0x08;
    OutputData &= 0xCF;
    break;

  case MarkParity:
    OutputData |= 0x28;
    break;

  case SpaceParity:
    OutputData |= 0x38;
    break;

  case NoParity:
  default:
    OutputData &= 0xF7;
  }

  IoWrite8 (
    (UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_LCR,
    (UINT8)(OutputData | B_UARY_LCR_DLAB)
    );

  //
  // Configure baud rate
  //
  IoWrite8 (
    (UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_BAUD_HIGH,
    (UINT8) (Divisor >> 8)
    );

  IoWrite8 (
    (UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_BAUD_LOW,
    (UINT8) (Divisor & 0xff)
    );

  //
  // Switch back to bank 0
  //
  IoWrite8 (
    (UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_LCR,
    OutputData
    );

  //
  // Enable FIFO, reset receive FIFO and transmit FIFO
  //
  IoWrite8 (
    (UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_FCR,
    (UINT8) (B_UARY_FCR_TRFIFIE | B_UARY_FCR_RESETRF | B_UARY_FCR_RESETTF)
    );

  return EFI_SUCCESS;
}

/**
  Write data to serial device.

  If the buffer is NULL, then return 0;
  if NumberOfBytes is zero, then return 0.

  @param  Buffer           Point of data buffer which need to be writed.
  @param  NumberOfBytes    Number of output bytes which are cached in Buffer.

  @retval 0                Write data failed.
  @retval !0               Actual number of bytes writed to serial device.

**/
UINTN
EFIAPI
SerialPortWrite (
  IN UINT8     *Buffer,
  IN UINTN     NumberOfBytes
)
{
  UINTN Result;
  UINT8 Data;
  BOOLEAN InterruptSet;

  if (NULL == Buffer) {
    return 0;
  }

  Result = NumberOfBytes;

  while (NumberOfBytes--) {

    //
    // Wait for the serail port to be ready, to make sure both the transmit FIFO
    // and shift register empty.
    //
    InterruptSet = GetInterruptState();
#ifdef __GNUC__
    __asm__ __volatile__(
      "cli \n"
    );
#else
    _asm {
      cli
    }
#endif

    do {
      Data = IoRead8 ((UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_LSR);
    } while ((Data & B_UART_LSR_TEMT) == 0);

    IoWrite8 ((UINT16) PcdGet64 (PcdSerialRegisterBase), *Buffer++);

    do {
      Data = IoRead8 ((UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_LSR);
    } while ((Data & B_UART_LSR_TEMT) == 0);

    SetInterruptState(InterruptSet);
  }

  return Result;
}


/*
  Read data from serial device and save the datas in buffer.

  If the buffer is NULL, then return 0;
  if NumberOfBytes is zero, then return 0.

  @param  Buffer           Point of data buffer which need to be writed.
  @param  NumberOfBytes    Number of output bytes which are cached in Buffer.

  @retval 0                Read data failed.
  @retval !0               Actual number of bytes raed to serial device.

**/
UINTN
EFIAPI
SerialPortRead (
  OUT UINT8     *Buffer,
  IN  UINTN     NumberOfBytes
)
{
  UINTN Result;
  UINT8 Data;
  BOOLEAN InterruptSet;

  if (NULL == Buffer) {
    return 0;
  }

  Result = NumberOfBytes;

  while (NumberOfBytes--) {
    //
    // Wait for the serail port to be ready.
    //
    InterruptSet = GetInterruptState();
#ifdef __GNUC__
    __asm__ __volatile__(
      "cli \n"
    );
#else
    _asm {
      cli
    }
#endif

    do {
      Data = IoRead8 ((UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_LSR);
    } while ((Data & B_UART_LSR_RXRDY) == 0);

    SetInterruptState(InterruptSet);

    *Buffer++ = IoRead8 ((UINT16) PcdGet64 (PcdSerialRegisterBase));
  }

  return Result;
}


/**
  Poll the serial device to see if there is any data waiting.

  If there is data waiting to be read from the serial port, then return
  TRUE.  If there is no data waiting to be read from the serial port, then
  return FALSE.

  @retval FALSE            There is no data waiting to be read.
  @retval TRUE             Data is waiting to be read.

**/
BOOLEAN
EFIAPI
SerialPortPoll (
  VOID
  )
{
  UINT8 Data;
  BOOLEAN InterruptSet;

  //
  // Read the serial port status
  //
  InterruptSet = GetInterruptState();
#ifdef __GNUC__
    __asm__ __volatile__(
      "cli \n"
    );
#else
    _asm {
      cli
    }
#endif

  Data = IoRead8 ((UINT16) PcdGet64 (PcdSerialRegisterBase) + R_UART_LSR);

  SetInterruptState(InterruptSet);

  return (BOOLEAN)((Data & B_UART_LSR_RXRDY) != 0);
}

