//
// This file contains 'Framework Code' and is licensed as such 
// under the terms of your license agreement with Intel or your
// vendor.  This file may not be modified, except as allowed by
// additional terms of your license agreement.                 
//
/** @file
  This library class provides functions and Macros to do read/erase/write/lock
  operation on FWH or compatible flash chips.

  
Copyright (c) 2006 Intel Corporation.<BR>
All rights reserved.  This software and associated documentation
(if any) is furnished under a license and may only be used or
copied in accordance with the terms of the license.  Except as
permitted by such license, no part of this software or
documentation may be reproduced, stored in a retrieval system, or 
transmitted in any form or by any means without the express written 
consent of Intel Corporation.

**/

#ifndef __FWHFLASH_LIB_H__
#define __FWHFLASH_LIB_H__

typedef struct {
  UINT32 BlockSize;    // size == 0 is last block
  UINT32 NumOfBlock;
} FWH_CHIP_LAYOUT;


/**
  Get the FWH flash layout: block size and block number.

  @param[out] Layout      The layout information of the flash chip.

  @retval     VOID
**/
VOID
EFIAPI
FwhFlashGetFlashLayout (
  FWH_CHIP_LAYOUT   **Layout
  );

/**
  This function did the initializaion works necessary before the Read, Write,
  EraseBlock, LockBlock functions can be called.

  @param[in]  VOID

  @retval      RETURN_SUCCESS       The function always returns RETURN_SUCCESS for now.
  @retval      RETURN_DEVICE_ERROR  If device error occurs.
**/
RETURN_STATUS 
EFIAPI
FwhFlashInit (
  VOID
  );


/**
  Read NumBytes bytes of data from the address specified by
  PAddress into Buffer.

  @param[in]      PAddress      The starting physical address of the read.
  @param[in,out]  NumBytes      On input, the number of bytes to read. On output, the number
                                of bytes actually read.
  @param[out]     Buffer        The destination data buffer for the read.

  @retval RETURN_SUCCESS.      Opertion is successful.
  @retval RETURN_DEVICE_ERROR  If there is any device errors.  
  
**/
RETURN_STATUS
EFIAPI
FwhFlashRead (
  IN     UINTN        PAddress,
  IN OUT UINTN        *NumBytes,
	OUT    UINT8        *Buffer
  );


/**
  Write NumBytes bytes of data from Buffer to the address specified by
  PAddresss.

  @param[in]      PAddress        The starting physical address of the write.
  @param[in,out]  NumBytes        On input, the number of bytes to write. On output,
                                  the actual number of bytes written.
  @param[in]      Buffer          The source data buffer for the write.

  @retval         RETURN_SUCCESS.      Opertion is successful.
  @retval         RETURN_DEVICE_ERROR  If there is any device errors.  

**/
RETURN_STATUS 
EFIAPI
FwhFlashWrite (
  IN      UINTN       PAddress,
  IN OUT  UINTN       *NumBytes,
	IN      UINT8       *Buffer
  );

/**
  Erase the block staring at PAddress.

  @param[in]  PAddress        The starting physical address of the region to be erased.

  @retval RETURN_SUCCESS.     Opertion is successful.
  @retval RETURN_DEVICE_ERROR If there is any device errors.  

**/
RETURN_STATUS 
EFIAPI
FwhFlashBlockErase (
  IN UINTN            PAddress
  );


/**
  Lock the block staring at PAddress so that write operation will not succeed if Lock is TRUE.
  Unlock the block for write otherwise.

  @param[in]  PAddress        The starting physical address of the region to be locked/unlocked.
                              The function only erase 1 block starting from PAddress.

  @param[in]  Lock            TRUE to lock; FALSE to unlock. This lock is mainly for WRITE and ERASE
                              protection. We didn't implement the READ lock for now.
  
  @retval RETURN_SUCCESS.      Opertion is successful.
  @retval RETURN_DEVICE_ERROR  If there is any device errors.  

**/
RETURN_STATUS
EFIAPI
FwhFlashBlockLock (
  IN  UINTN           PAddress, 
  IN  BOOLEAN         Lock
  );


#endif
