//
// This file contains 'Framework Code' and is licensed as such 
// under the terms of your license agreement with Intel or your
// vendor.  This file may not be modified, except as allowed by
// additional terms of your license agreement.                 
//
/** @file
  This library class provides functions and Macros to do read/erase/write/lock
  operation on SPI or compatible flash chips.

  
Copyright (c) 2011 Intel Corporation.<BR>
All rights reserved.  This software and associated documentation
(if any) is furnished under a license and may only be used or
copied in accordance with the terms of the license.  Except as
permitted by such license, no part of this software or
documentation may be reproduced, stored in a retrieval system, or 
transmitted in any form or by any means without the express written 
consent of Intel Corporation.

**/

#ifndef __SPIFLASH_LIB_H__
#define __SPIFLASH_LIB_H__

#define SPI_CHIP_COMMAND_WRITE_INDEX              0 
#define SPI_CHIP_COMMAND_READ_INDEX               1
#define SPI_CHIP_COMMAND_ERASE_INDEX              2
#define SPI_CHIP_COMMAND_ERASE_BLOCK_4K_INDEX     3
#define SPI_CHIP_COMMAND_READ_S_INDEX             4
#define SPI_CHIP_COMMAND_READ_ID_INDEX            5
#define SPI_CHIP_COMMAND_WRITE_S_INDEX            6
#define SPI_CHIP_COMMAND_WRITE_S_EN_INDEX         7
#define SPI_CHIP_COMMAND_WRITE_ENABLE_INDEX       8

#define SPI_CHIP_COMMAND_NUMBER                   9


typedef enum {
  SpiReadNoAdd,
  SpiReadAdd,
  SpiWriteNoAdd,
  SpiWriteAdd
} SPI_COMMAND_MODE;


typedef struct {
  UINT32 BlockSize;    // size == 0 is last block
  UINT32 NumOfBlock;
} SPI_CHIP_LAYOUT;


/**
  Get the number of SPI flash chip supported by a library
  instance.

  @param  VOID

  @retval UINTN   The number of flash supported by this 
                  library instance.

**/
UINTN
EFIAPI
SpiFlashGetSupportedChipNumber (
  IN      VOID
  );



/**
  Get SPI opcode list that must be loaded to a SPI host controller.
  
  @param[in]  Index       The index of the flash device to get the 
                          command list.
  @param[out] CommandList The SPI opcode list that must be loaded to
                          a host controller, i.e. the SPI host controler
                          on ICH.

  @retval RETURN_SUCCESS             Operation is successful.
  @retval RETURN_INVALID_PARAMETER   Index is greater than the number of SPI 
                                     flash chips supported by this library
                                     instance.
**/
RETURN_STATUS
EFIAPI
SpiFlashGetCommandList (
  IN      UINTN         Index,
  IN      UINT8         *CommandList
  );



/**
  Get the SPI flash layout: block size and block number.

  @param[in]  Index       The index of the flash device to get the 
                          flash layout.
  @param[out] Layout      The layout information of the flash chip.

  @retval RETURN_SUCCESS             Operation is successful.
  @retval RETURN_INVALID_PARAMETER   Index is greater than the number of SPI 
                                     flash chips supported by this library
                                     instance.
**/
RETURN_STATUS
EFIAPI
SpiFlashGetFlashLayout (
  UINTN             Index,
  SPI_CHIP_LAYOUT   **Layout
  );


/**
  Get flash device ID.

  @param[in]  Index       The index of the flash device to get the 
                          device list.
  @param[out] Id          The flash device ID.
  @param[out] IdWidth     The flash device ID width.
  @param[out] Attribute   The command mode when issuing the Read ID command. For some SPI chips,
                          a address cycle from SPI Host controoler is needed before device ID is
                          shift out from flash device, while for other flash device this address cycle
                          is not needed.
  
  @retval RETURN_SUCCESS             Operation is successful.
  @retval RETURN_INVALID_PARAMETER   Index is greater than the number of SPI 
                                     flash chips supported by this library
                                     instance.
**/
RETURN_STATUS
EFIAPI
SpiFlashGetDeviceId (
  IN    UINTN                 Index,
  OUT   UINTN                 *Id,
  OUT   UINTN                 *IdWidth,
  OUT   SPI_COMMAND_MODE      *Attribute
  );

#endif
