/*++
  This file contains 'Framework Code' and is licensed as such   
  under the terms of your license agreement with Intel or your  
  vendor.  This file may not be modified, except as allowed by  
  additional terms of your license agreement.                   
--*/
/** @file

  FWH Flash Chip Library instance for 82802A Intel flash chip.

Copyright (c) 2011, Intel Corporation. All rights reserved. <BR> 
This software and associated documentation (if any) is furnished
under a license and may only be used or copied in accordance
with the terms of the license. Except as permitted by such
license, no part of this software or documentation may be
reproduced, stored in a retrieval system, or transmitted in any
form or by any means without the express written consent of
Intel Corporation.

**/


#include "CommonHeader.h"

#include "FwhFlashChipIntel82802A.h"

//
// This flash layout imformation should match the information on data sheet.
//
STATIC FWH_CHIP_LAYOUT mFwhLayout[] = {
    {0x10000, 16}, // 64 Kbyte, 16 Blocks
    {0, 0}
};


/**
  This function did the initializaion works necessary before the Read, Write,
  EraseBlock, LockBlock functions can be called.

  @param[in]  VOID

  @retval      RETURN_SUCCESS       The function always returns EFI_SUCCESS for now.
  @retval      RETURN_DEVICE_ERROR  If device error occurs.
**/
RETURN_STATUS 
EFIAPI
FwhFlashInit (
  VOID
  )
{
  //
  // Currently, there is no implemenation for this function
  // for Intel82802a chip.
  //
  return RETURN_SUCCESS;
}


/**
  Get the FWH flash layout: block size and block number.

  @param[out] Layout      The layout information of the flash chip.

  @retval     VOID
**/
VOID
EFIAPI
FwhFlashGetFlashLayout (
  FWH_CHIP_LAYOUT   **Layout
  )
{
  ASSERT (Layout != NULL);
  
  *Layout = mFwhLayout;
}


/**
  Read NumBytes bytes of data from the address specified by
  PAddress into Buffer.

  @param[in]      PAddress      The starting physical address of the read.
  @param[in,out]  NumBytes      On input, the number of bytes to read. On output, the number
                                of bytes actually read.
  @param[out]     Buffer        The destination data buffer for the read.

  @retval RETURN_SUCCESS.      Opertion is successful.
  
**/
RETURN_STATUS
EFIAPI
FwhFlashRead (
  IN     UINTN                 PAddress,
  IN OUT UINTN                 *NumBytes,
  OUT    UINT8                 *Buffer
  )
{
  ASSERT (NULL != Buffer);
  ASSERT (NULL != NumBytes);

  CopyMem (Buffer, (VOID *) PAddress, *NumBytes);

  return RETURN_SUCCESS;

}



/**
  Busy loop until the FWH chip flag ready. The function will
  timeout if the FWH chip has no responce for too long.

  @param[in]      PAddress  The starting physical address of the status
                            check.

  @retval RETURN_SUCCESS.      FWH complete operations.
  @retval RETURN_DEVICE_ERROR  The wait timeouts.
  
**/
STATIC
RETURN_STATUS
WaitFwhUntilReady (
  UINTN       PAddress
  ) 
{
  UINTN       RetryTimes;
  RETURN_STATUS  Status;

  Status = RETURN_SUCCESS;
  RetryTimes = 0;
  
  while (RetryTimes < FVB_MAX_RETRY_TIMES) {

    MmioWrite8 (PAddress, FWH_READ_STATUS_COMMAND);
    if ((MmioRead8 (PAddress) & FWH_WRITE_STATE_STATUS) != 0) {
      //
      // Make sure that LOCK_DOWN and PROGRAM_STATUS both NOT SET.
      //
      ASSERT ((MmioRead8 (PAddress) & (FWH_LOCK_DOWN | FWH_PROGRAM_STATUS)) == 0);
      break;
    }

    RetryTimes++;
  }

  //
  // Device error if time out occurs
  //
  if (RetryTimes >= FVB_MAX_RETRY_TIMES) {
    Status    = RETURN_DEVICE_ERROR;
  }

  ASSERT (!RETURN_ERROR(Status));

  return Status;

}


/**
  Write NumBytes bytes of data from Buffer to the address specified by
  PAddresss.

  @param[in]      PAddress        The starting physical address of the write.
  @param[in,out]  NumBytes        On input, the number of bytes to write. On output,
                                  the actual number of bytes written.
  @param[in]      Buffer          The source data buffer for the write.

  @retval         RETURN_SUCCESS.     Opertion is successful.
                  RETURN_DEVICE_ERROR If there is device error.

**/
RETURN_STATUS 
EFIAPI
FwhFlashWrite (
  IN        UINTN         PAddress,
  IN OUT    UINTN         *NumBytes,
  IN        UINT8         *Buffer
  )
{
  UINT8       *Src;
  UINT8       *Dest;
  UINTN       Count;
  RETURN_STATUS  Status;

  ASSERT (NULL != Buffer);
  ASSERT (NULL != NumBytes);
  
  Status = RETURN_SUCCESS;

  //
  // Grab the lock before entering critical code section
  //
  //
  // Write data one byte at a time, don't write if the src and dest bytes match
  //
  Dest  = (UINT8 *) PAddress;
  Src   = Buffer;

  for (Count = 0; Count < *NumBytes; Count++, Dest++, Src++) {

    Status = WaitFwhUntilReady ((UINTN) Dest);
    if (RETURN_ERROR (Status)) {
      *NumBytes = Count;
      Status    = RETURN_DEVICE_ERROR;
      break;
    }

    MmioWrite8 ((UINTN) Dest, FWH_WRITE_SETUP_COMMAND);
    MmioWrite8 ((UINTN) Dest, *Src);

    Status = WaitFwhUntilReady ((UINTN) Dest);
    if (RETURN_ERROR (Status)) {
      *NumBytes = Count;
      Status    = RETURN_DEVICE_ERROR;
      break;
    }
  }
  //
  // Clear status register
  //
  MmioWrite8 (PAddress, FWH_CLEAR_STATUS_COMMAND);

  //
  // Issue read array command to return the FWH state machine to the
  // normal operational state
  //
  MmioWrite8 (PAddress, FWH_READ_ARRAY_COMMAND);

  return Status;
}


/**
  Erase 1 physical block staring at PAddress.

  @param[in]  PAddress        The starting physical address of the region to be erased.

  @retval     RETURN_SUCCESS.     Opertion is successful.
              RETURN_DEVICE_ERROR If there is device error.

**/
RETURN_STATUS 
EFIAPI
FwhFlashBlockErase (
  IN UINTN              PAddress
  )
{
  RETURN_STATUS  Status;

  Status = WaitFwhUntilReady (PAddress);
  if (RETURN_ERROR (Status)) {
    return Status;
  }
  //
  // Send erase commands to FWH
  //
  MmioWrite8 (PAddress, FWH_BLOCK_ERASE_SETUP_COMMAND);
  MmioWrite8 (PAddress, FWH_BLOCK_ERASE_CONFIRM_COMMAND);
  

  Status = WaitFwhUntilReady (PAddress);

  //
  // Clear status register
  //
  MmioWrite8 (PAddress, FWH_CLEAR_STATUS_COMMAND);

  //
  // Issue read array command to return the FWH state machine to the normal op state
  //
  MmioWrite8 (PAddress, FWH_READ_ARRAY_COMMAND);

  return Status;
}



/**
  Lock the block staring at PAddress.

  @param[in]  PAddress        The starting physical address of the region to be locked/unlocked.
                              The function only lock 1 physical block starting from PAddress.
                              The block map information is returned by FwhFlashGetFlashLayout
                              function.
  @param[in]  Lock            TRUE to lock; FALSE to unlock. This lock is mainly for WRITE and ERASE
                              protection. We didn't implement the READ lock for now.
  
  @retval RETURN_SUCCESS.     Opertion is successful.
          RETURN_DEVICE_ERROR If there is device error.

**/
RETURN_STATUS
EFIAPI
FwhFlashBlockLock (
  IN  UINTN                   PAddress, 
  IN  BOOLEAN                 Lock
  )
{
  UINT8   LockStatus;
  UINTN   LockRegister;

  LockRegister = PAddress + 2;

  LockStatus = MmioRead8 (LockRegister);

  //
  // According to Intel 82802AB/82802AC Firmware Hub (FWH) spec,
  // If the block is locked down, the block will remain locked-down
  // until reset (with RST# or INIT#), or until the device is
  // power-cycled.
  //
  // Flash is locked down. There is nothing we can do. Just return 
  // RETURN_DEVICE_ERROR for now.
  //
  if ((LockStatus & FWH_LOCK_DOWN) == FWH_LOCK_DOWN) {
    return RETURN_DEVICE_ERROR;
  }

  if (Lock) {
    MmioOr8 (LockRegister, FWH_WRITE_LOCK);
    ASSERT ((MmioRead8 (LockRegister) & FWH_WRITE_LOCK) != 0);
  } else {
    MmioAnd8 (LockRegister, (UINT8) ~FWH_WRITE_LOCK);
    ASSERT ((MmioRead8 (LockRegister) & FWH_WRITE_LOCK) == 0);
  }
  return RETURN_SUCCESS;
}

