/*++
  This file contains 'Framework Code' and is licensed as such   
  under the terms of your license agreement with Intel or your  
  vendor.  This file may not be modified, except as allowed by  
  additional terms of your license agreement.                   
--*/
/** @file 

  SPI Flash Chip Library Instance for AT25DF641 flash chip.

Copyright (c) 2011, Intel Corporation. All rights reserved. <BR> 
This software and associated documentation (if any) is furnished
under a license and may only be used or copied in accordance
with the terms of the license. Except as permitted by such
license, no part of this software or documentation may be
reproduced, stored in a retrieval system, or transmitted in any
form or by any means without the express written consent of
Intel Corporation.

**/

#include "SpiFlashChipAT25DF641.h"

STATIC UINT8 mSpiAt25DF641_CommandList[SPI_CHIP_COMMAND_NUMBER] = {
  SPI_AT25DF641_COMMAND_WRITE,
  SPI_AT25DF641_COMMAND_READ,
  SPI_AT25DF641_COMMAND_ERASE_32KBYTE,        // Must match mSpiLayout[0].BlockSize
  SPI_AT25DF641_COMMAND_ERASE_4KBYTE,
  SPI_AT25DF641_COMMAND_READ_STATUS,
  SPI_AT25DF641_COMMAND_READ_ID,
  SPI_AT25DF641_COMMAND_WRITE_S,
  SPI_AT25DF641_COMMAND_WRITE_S_EN,
  SPI_AT25DF641_COMMAND_WRITE_ENABLE
};

STATIC SPI_CHIP_LAYOUT mSpiLayout[] = {
    {0x8000, 256}, // 32Kbyte/Block, 256 Blocks
    {0, 0}
};


/**
  Get the number of SPI flash chip supported by a library
  instance.

  @param  VOID

  @retval UINTN   The number of flash supported by this 
                  library instance.

**/
UINTN
EFIAPI
SpiFlashGetSupportedChipNumber (
  VOID
  )
{
  return 1;
}


/**
  Get SPI opcode list that must be loaded to a SPI host controller.
  
  @param[in]  Index       The index of the flash device to get the 
                          command list.
  @param[out] CommandList The SPI opcode list that must be loaded to
                          a host controller, i.e. the SPI host controler
                          on ICH.

  @retval RETURN_SUCCESS             Operation is successful.
  @retval RETURN_INVALID_PARAMETER   Index is greater than the number of SPI 
                                     flash chips supported by this library
                                     instance.
**/
RETURN_STATUS
EFIAPI
SpiFlashGetCommandList (
  UINTN   Index,
  UINT8   *CommandList
  )
{
  if (Index > 0) {
    return RETURN_INVALID_PARAMETER;
  }

  CopyMem (CommandList, mSpiAt25DF641_CommandList, sizeof(mSpiAt25DF641_CommandList));

  return RETURN_SUCCESS;
}


/**
  Get the SPI flash layout: block size and block number.

  @param[in]  Index       The index of the flash device to get the 
                          flash layout.
  @param[out] Layout      The layout information of the flash chip.

  @retval RETURN_SUCCESS             Operation is successful.
  @retval RETURN_INVALID_PARAMETER   Index is greater than the number of SPI 
                                     flash chips supported by this library
                                     instance.
**/
RETURN_STATUS
EFIAPI
SpiFlashGetFlashLayout (
  IN    UINTN             Index,
  OUT   SPI_CHIP_LAYOUT   **Layout
  )
{
  ASSERT (Layout != NULL);
  
  if (Index > 0) {
    return RETURN_INVALID_PARAMETER;
  }
  
  *Layout = mSpiLayout;
  
  return RETURN_SUCCESS;
}


/**
  Get flash device ID.
  If Id, IdWidth or Attribute is NULL, then ASSERT.

  @param[in]  Index       The index of the flash device to get the 
  
                          device list.
  @param[out] Id          The flash device ID.
  @param[out] IdWidth     The flash device ID width.
  @param[out] Attribute   The command mode when issuing the Read ID command. For some SPI chips,
                          a address cycle from SPI Host controoler is needed before device ID is
                          shift out from flash device, while for other flash device this address cycle
                          is not needed.
  
  @retval RETURN_SUCCESS             Operation is successful.
  @retval RETURN_INVALID_PARAMETER   Index is greater than the number of SPI 
                                     flash chips supported by this library
                                     instance.
**/
RETURN_STATUS
EFIAPI
SpiFlashGetDeviceId (
  IN    UINTN                 Index,
  OUT   UINTN                 *Id,
  OUT   UINTN                 *IdWidth,
  OUT   SPI_COMMAND_MODE      *Attribute
  )
{
  ASSERT (NULL != Id);
  ASSERT (NULL != IdWidth);
  ASSERT (NULL != Attribute);

  //
  // This library instance ONLY support a single SPI flash.
  //
  if (Index > 0) {
    return RETURN_INVALID_PARAMETER;
  }


  *Id = 0x481F;
  *IdWidth = 2;
  *Attribute = SpiReadNoAdd;

  return RETURN_SUCCESS;
}